@@ -80,8 +80,8 @@ wrap_patch(git_patch *patch)
         py_patch->status = git_diff_status_char(delta->status);
         py_patch->similarity = delta->similarity;
         py_patch->flags = delta->flags;
-        py_patch->old_oid = git_oid_allocfmt(&delta->old_file.oid);
-        py_patch->new_oid = git_oid_allocfmt(&delta->new_file.oid);
+        py_patch->old_id = git_oid_allocfmt(&delta->old_file.id);
+        py_patch->new_id = git_oid_allocfmt(&delta->new_file.id);
 
         git_patch_line_stats(NULL, &additions, &deletions, patch);
         py_patch->additions = additions;
@@ -149,8 +149,8 @@ static void
 Patch_dealloc(Patch *self)
 {
     Py_CLEAR(self->hunks);
-    free(self->old_oid);
-    free(self->new_oid);
+    free(self->old_id);
+    free(self->new_id);
     /* We do not have to free old_file_path and new_file_path, they will
      * be freed by git_diff_list_free in Diff_dealloc */
     PyObject_Del(self);
@@ -159,8 +159,8 @@ Patch_dealloc(Patch *self)
 PyMemberDef Patch_members[] = {
     MEMBER(Patch, old_file_path, T_STRING, "old file path"),
     MEMBER(Patch, new_file_path, T_STRING, "new file path"),
-    MEMBER(Patch, old_oid, T_STRING, "old oid"),
-    MEMBER(Patch, new_oid, T_STRING, "new oid"),
+    MEMBER(Patch, old_id, T_STRING, "old oid"),
+    MEMBER(Patch, new_id, T_STRING, "new oid"),
     MEMBER(Patch, status, T_CHAR, "status"),
     MEMBER(Patch, similarity, T_INT, "similarity"),
     MEMBER(Patch, hunks, T_OBJECT, "hunks"),
@@ -292,8 +292,7 @@ PyObject *
 Diff_patch__get__(Diff *self)
 {
     git_patch* patch;
-    char **strings = NULL;
-    char *buffer = NULL;
+    git_buf buf = {NULL};
     int err = GIT_ERROR;
     size_t i, len, num;
     PyObject *py_patch = NULL;
@@ -301,32 +300,25 @@ Diff_patch__get__(Diff *self)
     num = git_diff_num_deltas(self->list);
     if (num == 0)
         Py_RETURN_NONE;
-    MALLOC(strings, num * sizeof(char*), cleanup);
 
     for (i = 0, len = 1; i < num ; ++i) {
         err = git_patch_from_diff(&patch, self->list, i);
         if (err < 0)
             goto cleanup;
 
-        err = git_patch_to_str(&(strings[i]), patch);
+        /* This appends to the current buf, so we can simply keep passing it */
+        err = git_patch_to_buf(&buf, patch);
         if (err < 0)
             goto cleanup;
 
-        len += strlen(strings[i]);
         git_patch_free(patch);
     }
 
-    CALLOC(buffer, (len + 1), sizeof(char), cleanup);
-    for (i = 0; i < num; ++i) {
-        strcat(buffer, strings[i]);
-        free(strings[i]);
-    }
-    free(strings);
-
-    py_patch = to_unicode(buffer, NULL, NULL);
-    free(buffer);
+    py_patch = to_unicode(buf.ptr, NULL, NULL);
+    git_buf_free(&buf);
 
 cleanup:
+    git_buf_free(&buf);
     return (err < 0) ? Error_set(err) : py_patch;
 }
 
