@@ -45,7 +45,7 @@ extern PyTypeObject RepositoryType;
 int
 Index_init(Index *self, PyObject *args, PyObject *kwds)
 {
-    char *path;
+    char *path = NULL;
     int err;
 
     if (kwds && PyDict_Size(kwds) > 0) {
@@ -53,9 +53,10 @@ Index_init(Index *self, PyObject *args, PyObject *kwds)
         return -1;
     }
 
-    if (!PyArg_ParseTuple(args, "s", &path))
+    if (!PyArg_ParseTuple(args, "|s", &path))
         return -1;
 
+    self->repo = NULL;
     err = git_index_open(&self->index, path);
     if (err < 0) {
         Error_set_str(err, path);
@@ -425,26 +426,46 @@ Index_remove(Index *self, PyObject *args)
 PyDoc_STRVAR(Index_read_tree__doc__,
   "read_tree(tree)\n"
   "\n"
-  "Update the index file from the tree identified by the given oid.");
+  "Update the index file from the specified tree. The tree can be a Tree object or an Oid.\n"
+  "Using an Oid is only possible if this index is associated with a repository");
 
 PyObject *
 Index_read_tree(Index *self, PyObject *value)
 {
     git_oid oid;
-    git_tree *tree;
-    int err;
+    git_tree *tree = NULL;
+    int err, need_free = 0;
     size_t len;
 
     len = py_oid_to_git_oid(value, &oid);
-    if (len == 0)
-        return NULL;
+    if (len == 0) {
+        Tree *py_tree;
+        if (!PyObject_TypeCheck(value, &TreeType)) {
+            return NULL;
+        }
 
-    err = git_tree_lookup_prefix(&tree, self->repo->repo, &oid, len);
-    if (err < 0)
-        return Error_set(err);
+        PyErr_Clear();
+        py_tree = (Tree *) value;
+        tree = py_tree->tree;
+    }
+
+    /*
+     * if the user passed in an id but we're not associated with a
+     * repo, we can't do anything
+     */
+    if (tree == NULL && self->repo == NULL) {
+        PyErr_SetString(PyExc_TypeError, "id given but no associated repository");
+        return NULL;
+    } else if (tree == NULL) {
+        need_free = 1;
+        err = git_tree_lookup_prefix(&tree, self->repo->repo, &oid, len);
+        if (err < 0)
+            return Error_set(err);
+    }
 
     err = git_index_read_tree(self->index, tree);
-    git_tree_free(tree);
+    if (need_free)
+        git_tree_free(tree);
     if (err < 0)
         return Error_set(err);
 
@@ -628,7 +649,7 @@ IndexEntry_init(IndexEntry *self, PyObject *args, PyObject *kwds)
         return -1;
 
     if (id)
-        git_oid_cpy(&self->entry.oid, &id->oid);
+        git_oid_cpy(&self->entry.id, &id->oid);
 
     if (mode)
         self->entry.mode = mode;
@@ -689,18 +710,18 @@ IndexEntry_path__set__(IndexEntry *self, PyObject *py_path)
     return 0;
 }
 
-PyDoc_STRVAR(IndexEntry_oid__doc__, "Object id.");
+PyDoc_STRVAR(IndexEntry_id__doc__, "Object id.");
 
 PyObject *
-IndexEntry_oid__get__(IndexEntry *self)
+IndexEntry_id__get__(IndexEntry *self)
 {
-    return git_oid_to_python(&self->entry.oid);
+    return git_oid_to_python(&self->entry.id);
 }
 
 int
-IndexEntry_oid__set__(IndexEntry *self, PyObject *py_id)
+IndexEntry_id__set__(IndexEntry *self, PyObject *py_id)
 {
-    if (!py_oid_to_git_oid(py_id, &self->entry.oid))
+    if (!py_oid_to_git_oid(py_id, &self->entry.id))
         return -1;
 
     return 0;
@@ -711,13 +732,13 @@ PyDoc_STRVAR(IndexEntry_hex__doc__, "Hex id.");
 PyObject *
 IndexEntry_hex__get__(IndexEntry *self)
 {
-    return git_oid_to_py_str(&self->entry.oid);
+    return git_oid_to_py_str(&self->entry.id);
 }
 
 PyGetSetDef IndexEntry_getseters[] = {
     GETSET(IndexEntry, mode),
     GETSET(IndexEntry, path),
-    GETSET(IndexEntry, oid),
+    GETSET(IndexEntry, id),
     GETTER(IndexEntry, hex),
     {NULL},
 };
