@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2013 The pygit2 contributors
+ * Copyright 2010-2014 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -45,15 +45,15 @@ Object_dealloc(Object* self)
 {
     Py_CLEAR(self->repo);
     git_object_free(self->obj);
-    PyObject_Del(self);
+    Py_TYPE(self)->tp_free(self);
 }
 
 
-PyDoc_STRVAR(Object_oid__doc__,
+PyDoc_STRVAR(Object_id__doc__,
     "The object id, an instance of the Oid type.");
 
 PyObject *
-Object_oid__get__(Object *self)
+Object_id__get__(Object *self)
 {
     const git_oid *oid;
 
@@ -63,10 +63,20 @@ Object_oid__get__(Object *self)
     return git_oid_to_python(oid);
 }
 
+PyDoc_STRVAR(Object_oid__doc__,
+    "The object id, an instance of the Oid type.\n"
+    "This attribute is deprecated, please use 'id'\n");
+
+PyObject *
+Object_oid__get__(Object *self)
+{
+    return Object_id__get__(self);
+}
 
 PyDoc_STRVAR(Object_hex__doc__,
     "Hexadecimal representation of the object id. This is a shortcut for\n"
-    "Object.oid.hex");
+    "Object.oid.hex\n"
+    "This attribute is deprecated, please use 'id'\n");
 
 PyObject *
 Object_hex__get__(Object *self)
@@ -117,8 +127,58 @@ Object_read_raw(Object *self)
     return aux;
 }
 
+static git_otype
+py_type_to_git_type(PyTypeObject *py_type)
+{
+    git_otype type = GIT_OBJ_BAD;
+
+    if (py_type == &CommitType) {
+        type = GIT_OBJ_COMMIT;
+    } else if (py_type == &TreeType) {
+        type = GIT_OBJ_TREE;
+    } else if (py_type == &BlobType) {
+        type = GIT_OBJ_BLOB;
+    } else if (py_type == &TagType) {
+        type = GIT_OBJ_TAG;
+    }
+
+    return type;
+}
+
+PyDoc_STRVAR(Object_peel__doc__,
+  "peel(target_type) -> Object\n"
+  "\n"
+  "Peel the current object and returns the first object of the given type\n");
+
+PyObject *
+Object_peel(Object *self, PyObject *py_type)
+{
+    int type = -1, err;
+    git_object *peeled;
+
+    if (PyLong_Check(py_type)) {
+        type = PyLong_AsLong(py_type);
+        if (type == -1 && PyErr_Occurred())
+            return NULL;
+    } else if (PyType_Check(py_type)) {
+        type = py_type_to_git_type((PyTypeObject *) py_type);
+    }
+
+    if (type == -1) {
+        PyErr_SetString(PyExc_ValueError, "invalid target type");
+        return NULL;
+    }
+
+    err = git_object_peel(&peeled, self->obj, (git_otype)type);
+    if (err < 0)
+        return Error_set(err);
+
+    return wrap_object(peeled, self->repo);
+}
+
 PyGetSetDef Object_getseters[] = {
     GETTER(Object, oid),
+    GETTER(Object, id),
     GETTER(Object, hex),
     GETTER(Object, type),
     {NULL}
@@ -126,6 +186,7 @@ PyGetSetDef Object_getseters[] = {
 
 PyMethodDef Object_methods[] = {
     METHOD(Object, read_raw, METH_NOARGS),
+    METHOD(Object, peel, METH_O),
     {NULL}
 };
 
