@@ -40,7 +40,9 @@ from os.path import join, realpath
 
 # Import from pygit2
 from pygit2 import GIT_OBJ_ANY, GIT_OBJ_BLOB, GIT_OBJ_COMMIT
-from pygit2 import init_repository, clone_repository, discover_repository
+from pygit2 import GIT_MERGE_ANALYSIS_NONE, GIT_MERGE_ANALYSIS_NORMAL, GIT_MERGE_ANALYSIS_UP_TO_DATE
+from pygit2 import GIT_MERGE_ANALYSIS_FASTFORWARD, GIT_MERGE_ANALYSIS_UNBORN
+from pygit2 import init_repository, clone_repository, clone_into, discover_repository
 from pygit2 import Oid, Reference, hashfile
 import pygit2
 from . import utils
@@ -68,6 +70,15 @@ class RepositoryTest(utils.BareRepoTestCase):
         self.assertFalse(self.repo.head_is_unborn)
         self.assertFalse(self.repo.head_is_detached)
 
+    def test_set_head(self):
+        # Test setting a detatched HEAD.
+        self.repo.head = Oid(hex=PARENT_SHA)
+        self.assertEqual(self.repo.head.target.hex, PARENT_SHA)
+        # And test setting a normal HEAD.
+        self.repo.head = "refs/heads/master"
+        self.assertEqual(self.repo.head.name, "refs/heads/master")
+        self.assertEqual(self.repo.head.target.hex, HEAD_SHA)
+
     def test_read(self):
         self.assertRaises(TypeError, self.repo.read, 123)
         self.assertRaisesWithArg(KeyError, '1' * 40, self.repo.read, '1' * 40)
@@ -188,6 +199,11 @@ class RepositoryTest_II(utils.RepoTestCase):
         expected = realpath(self.repo_path)
         self.assertEqual(directory, expected)
 
+    def test_set_workdir(self):
+        directory = tempfile.mkdtemp()
+        self.repo.workdir = directory
+        self.assertEqual(realpath(self.repo.workdir), realpath(directory))
+
     def test_checkout_ref(self):
         ref_i18n = self.repo.lookup_reference('refs/heads/i18n')
 
@@ -308,57 +324,50 @@ class RepositoryTest_III(utils.RepoTestCaseForMerging):
     def test_merge_none(self):
         self.assertRaises(TypeError, self.repo.merge, None)
 
-    def test_merge_uptodate(self):
+    def test_merge_analysis_uptodate(self):
         branch_head_hex = '5ebeeebb320790caf276b9fc8b24546d63316533'
-        branch_oid = self.repo.get(branch_head_hex).id
-        merge_result = self.repo.merge(branch_oid)
-        self.assertTrue(merge_result.is_uptodate)
-        self.assertFalse(merge_result.is_fastforward)
-        self.assertEqual(None, merge_result.fastforward_oid)
+        branch_id = self.repo.get(branch_head_hex).id
+        analysis, preference = self.repo.merge_analysis(branch_id)
+
+        self.assertTrue(analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE)
+        self.assertFalse(analysis & GIT_MERGE_ANALYSIS_FASTFORWARD)
         self.assertEqual({}, self.repo.status())
 
-    def test_merge_fastforward(self):
+    def test_merge_analysis_fastforward(self):
         branch_head_hex = 'e97b4cfd5db0fb4ebabf4f203979ca4e5d1c7c87'
-        branch_oid = self.repo.get(branch_head_hex).id
-        merge_result = self.repo.merge(branch_oid)
-        self.assertFalse(merge_result.is_uptodate)
-        self.assertTrue(merge_result.is_fastforward)
-        # Asking twice to assure the reference counting is correct
-        self.assertEqual(branch_head_hex, merge_result.fastforward_oid.hex)
-        self.assertEqual(branch_head_hex, merge_result.fastforward_oid.hex)
+        branch_id = self.repo.get(branch_head_hex).id
+        analysis, preference = self.repo.merge_analysis(branch_id)
+        self.assertFalse(analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE)
+        self.assertTrue(analysis & GIT_MERGE_ANALYSIS_FASTFORWARD)
         self.assertEqual({}, self.repo.status())
 
     def test_merge_no_fastforward_no_conflicts(self):
         branch_head_hex = '03490f16b15a09913edb3a067a3dc67fbb8d41f1'
-        branch_oid = self.repo.get(branch_head_hex).id
-        merge_result = self.repo.merge(branch_oid)
-        self.assertFalse(merge_result.is_uptodate)
-        self.assertFalse(merge_result.is_fastforward)
+        branch_id = self.repo.get(branch_head_hex).id
+        analysis, preference = self.repo.merge_analysis(branch_id)
+        self.assertFalse(analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE)
+        self.assertFalse(analysis & GIT_MERGE_ANALYSIS_FASTFORWARD)
         # Asking twice to assure the reference counting is correct
-        self.assertEqual(None, merge_result.fastforward_oid)
-        self.assertEqual(None, merge_result.fastforward_oid)
-        self.assertEqual({'bye.txt': 1}, self.repo.status())
-        self.assertEqual({'bye.txt': 1}, self.repo.status())
-        # Checking the index works as expected
-        self.repo.index.remove('bye.txt')
-        self.repo.index.write()
-        self.assertEqual({'bye.txt': 128}, self.repo.status())
+        self.assertEqual({}, self.repo.status())
+        self.assertEqual({}, self.repo.status())
 
     def test_merge_no_fastforward_conflicts(self):
         branch_head_hex = '1b2bae55ac95a4be3f8983b86cd579226d0eb247'
-        branch_oid = self.repo.get(branch_head_hex).id
-        merge_result = self.repo.merge(branch_oid)
-        self.assertFalse(merge_result.is_uptodate)
-        self.assertFalse(merge_result.is_fastforward)
+        branch_id = self.repo.get(branch_head_hex).id
+
+        analysis, preference = self.repo.merge_analysis(branch_id)
+        self.assertFalse(analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE)
+        self.assertFalse(analysis & GIT_MERGE_ANALYSIS_FASTFORWARD)
+
+        self.repo.merge(branch_id)
+        status = pygit2.GIT_STATUS_WT_NEW | pygit2.GIT_STATUS_INDEX_DELETED
         # Asking twice to assure the reference counting is correct
-        self.assertEqual(None, merge_result.fastforward_oid)
-        self.assertEqual(None, merge_result.fastforward_oid)
-        self.assertEqual({'.gitignore': 132}, self.repo.status())
-        self.assertEqual({'.gitignore': 132}, self.repo.status())
+        self.assertEqual({'.gitignore': status}, self.repo.status())
+        self.assertEqual({'.gitignore': status}, self.repo.status())
         # Checking the index works as expected
         self.repo.index.add('.gitignore')
         self.repo.index.write()
-        self.assertEqual({'.gitignore': 2}, self.repo.status())
+        self.assertEqual({'.gitignore': pygit2.GIT_STATUS_INDEX_MODIFIED}, self.repo.status())
 
     def test_merge_invalid_hex(self):
         branch_head_hex = '12345678'
@@ -461,6 +470,13 @@ class CloneRepositoryTest(utils.NoRepoTestCase):
         self.assertFalse(repo.is_empty)
         self.assertEqual(repo.remotes[0].name, "custom_remote")
 
+    def test_clone_into(self):
+        repo_path = "./test/data/testrepo.git/"
+        repo = init_repository(os.path.join(self._temp_dir, "clone-into"))
+        remote = repo.create_remote("origin", 'file://' + os.path.realpath(repo_path))
+        clone_into(repo, remote)
+        self.assertTrue('refs/remotes/origin/master' in repo.listall_references())
+
     def test_clone_with_credentials(self):
         credentials = pygit2.UserPass("libgit2", "libgit2")
         repo = clone_repository(
